/*
Copyright 2013, the Dart project authors.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above
      copyright notice, this list of conditions and the following
      disclaimer in the documentation and/or other materials provided
      with the distribution.
    * Neither the name of Google LLC nor the names of its
      contributors may be used to endorse or promote products derived
      from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

part of 'parser.dart';

/// The base type for all nodes in a CSS abstract syntax tree.
abstract class TreeNode {
  /// Classic double-dispatch visitor for implementing passes.
  dynamic visit(Visitor visitor) {}
}

class Identifier extends TreeNode {
  String name;

  Identifier(this.name) : super();

  @override
  String toString() {
    // Try to use the identifier's original lexeme to preserve any escape codes
    // as authored. The name, which may include escaped values, may no longer be
    // a valid identifier.
    return name;
  }

  bool isFunction() {
    return name == 'var' ||
        name == 'rgb' ||
        name == 'rgba' ||
        name == 'translate' ||
        name == 'rotate' ||
        name == 'calc' ||
        name == 'hsl' ||
        name == 'hsla' ||
        name == 'linear-gradient' ||
        name == 'radial-gradient' ||
        name == 'repeating-linear-gradient' ||
        name == 'repeating-radial-gradient' ||
        name == 'cubic-bezier' ||
        name == 'attr' ||
        name == 'url';
  }
}

class Wildcard extends TreeNode {
  Wildcard() : super();

  String get name => '*';
}

class ThisOperator extends TreeNode {
  ThisOperator() : super();

  String get name => '&';
}

class Negation extends TreeNode {
  Negation() : super();

  String get name => 'not';
}
